(function($) {

	window.CrayonTagEditor = new function() {
		var base = this;
		
		var loaded = false;
		var editing = false;
		var insertCallback, editCallback, showCallback, hideCallback = null;
		// Used for encoding, decoding
		var inputHTML, outputHTML, editor_name, ajax_class_timer = null;
		var ajax_class_timer_count = 0;
		
		var code_refresh, url_refresh = null;
		
		// Current $ obj of pre node
		var currCrayon = null;
		// Classes from pre node, excl. settings
		var currClasses = '';
		// Whether to make span or pre
		var is_inline = false;
		
		// Generated in WP and contains the settings
		var s, gs, util = null;
		// For use in async functions
		var me = this;
		
		// CSS
		var dialog, code, clear, submit = null;
		
		base.init = function(button) {
			s = CrayonTagEditorSettings;
			gs = CrayonSyntaxSettings;
			util = CrayonUtil;

			base.loadDialog();
			$(button).crayonFancybox({
        		href : s.content_css,
        		margin : [40,10,40,10],
        		padding : 0,
        		width : 690,
        		height : '100%',
        		autoSize : false,
        		title : '',
        		beforeShow : function () {
            		$(this.outer).prepend($(s.bar_content));
        		},
        		afterShow : function () {
        			$(s.code_css).focus();
        		},
        		beforeClose : function () {
        			$(s.bar).prepend($(s.bar_content));
        		},
        		closeBtn : false
        	});
			
			$(s.cancel_css).live('click', function () {
				$.crayonFancybox.close();
				return false;
			});
		};
		
		base.hide = function() {
    		$.crayonFancybox.close();
			return false;
		};
		
		// XXX Loads dialog contents
	    base.loadDialog = function() {
	    	// Loaded once url is given
	    	if (!loaded) {
	    		loaded = true;
	    	} else {
	    		return;
	    	}
	    	
	        // Load the editor content 
            $.get(gs.ajaxurl, {action : 'crayon-tag-editor'}, function(data) {
	        	dialog = $('<div id="'+s.css+'"></div>');
	            dialog.appendTo('body').hide();
	        	dialog.html(data);
	        	
	        	me.setOrigValues();
	        	
	        	submit = dialog.find(s.submit_css);
	        	
	        	code = $(s.code_css);
	        	clear = $('#crayon-te-clear');
	        	code_refresh = function () {
	        		var clear_visible = clear.is(":visible");
	        		if (code.val().length > 0 && !clear_visible) {
	        			clear.show();
	        			code.removeClass(gs.selected);
	        		} else if (code.val().length <= 0) {
	        			clear.hide();
	        		}
	        	};
	
	        	code.keyup(code_refresh);
	        	code.change(code_refresh);
	        	clear.click(function() {
	        		code.val('');
	        		code.removeClass(gs.selected);
	        		code.focus();
	        	});
	        	
	        	var url = $(s.url_css);
	        	var url_info = $(s.url_info_css);
	        	var exts = CrayonTagEditorSettings.extensions;
	        	url_refresh = function () {
	        		if (url.val().length > 0 && !url_info.is(":visible")) {
	        			url_info.show();
	        			url.removeClass(gs.selected);
	        		} else if (url.val().length <= 0) {
	        			url_info.hide();
	        		}
	        		
	        		// Check for extensions and select language automatically
	        		var ext = CrayonSyntaxUtil.getExt(url.val());
	        		if (ext) {
	        			var lang = exts[ext];
	        			// Otherwise use the extention as the lang
	        			var lang_id = lang ? lang : ext;
	        			var final_lang = CrayonTagEditorSettings.fallback_lang;
	        			$(s.lang_css + ' option').each(function() {
	        				if ($(this).val() == lang_id) {
	        					final_lang = lang_id; 
	        				}
	        			});
	        			$(s.lang_css).val(final_lang);
	        		}
	        	};
	        	url.keyup(url_refresh);
	        	url.change(url_refresh);
	        	
	        	var setting_change = function() {
	    			var setting = $(this);
	        		var orig_value = $(this).attr(gs.orig_value);
	        		if (typeof orig_value == 'undefined') {
	        			orig_value = '';
	        		}
	        		// Depends on type
	        		var value = me.settingValue(setting);
	        		console_log(setting.attr('id') + ' value: ' + value);
	        		var highlight = null;
	        		if (setting.is('input[type=checkbox]')) {
	    				highlight = setting.next('span');
	    			}
	    			
	        		console_log('   >>> ' + setting.attr('id') + ' is ' + orig_value + ' = ' + value);
	        		if (orig_value == value) {
	        			// No change
	    				setting.removeClass(gs.changed);
	    				if (highlight) {
	    					highlight.removeClass(gs.changed);
	    				}
	     			} else {
	    				// Changed
	     				setting.addClass(gs.changed);
	     				if (highlight) {
	     					highlight.addClass(gs.changed);
	     				}
	    			}
	    			// Save standardized value for later
	    			me.settingValue(setting, value);
	    		};
	        	$('.'+gs.setting+'[id]:not(.'+gs.special+')').each(function() {
	        		$(this).change(setting_change);
	        		$(this).keyup(setting_change);
	        	});
	        });
	    };
	    
	    // XXX Displays the dialog.
		base.showDialog = function(insert, edit, show, hide, editor_str, ed, node, input, output) {
			// Need to reset all settings back to original, clear yellow highlighting
			me.resetSettings();
			// Save these for when we add a Crayon
			insertCallback = insert;
	    	editCallback = edit;
	    	showCallback = show;
	    	hideCallback = hide;
	    	inputHTML = input;
	    	outputHTML = output;
	    	editor_name = editor_str;
			// If we have selected a Crayon, load in the contents
			// TODO put this in a separate function
			var currNode = null;
			is_inline = false;
			if (typeof node != 'undefined' && node != null) {
				currNode = node;
			} else {
				// Get it from editor selection, not as precise
				currNode = ed != null ? ed.selection.getNode() : null;
			}
			
	    	// Unbind submit
	    	submit.unbind();
	    	submit.click(function() {
	    		me.submitButton();
	    		return false;
	    	});
	    	me.setSubmitText(s.submit_add);
			
			if (me.isCrayon(currNode)) {
				currCrayon = $(currNode); 
				if (currCrayon.length != 0) {
					// Read back settings for editing
					currClasses = currCrayon.attr('class');
					var re = new RegExp('\\b([A-Za-z-]+)'+s.attr_sep+'(\\S+)', 'gim');
					var matches = re.execAll(currClasses);
					// Retain all other classes, remove settings
					currClasses = $.trim(currClasses.replace(re, ''));
					var atts = {};
					for (var i in matches) {
						var id = matches[i][1];
						var value = matches[i][2];
						atts[id] = value;
					}
					
					// Title
					var title = currCrayon.attr('title');
					if (title) {
						atts['title'] = title;
					}
					
					// URL
					var url = currCrayon.attr('data-url');
					if (url) {
						atts['url'] = url;
					}
					
					// Inverted settings
					if (typeof atts['highlight'] != 'undefined') {
						atts['highlight'] = '0' ? '1' : '0';
					}
	
					// Inline
					is_inline = currCrayon.hasClass(s.inline_css);
					atts['inline'] = is_inline ? '1' : '0';
					
					// Ensure language goes to fallback if invalid
					var avail_langs = [];
					$(s.lang_css + ' option').each(function(){
						var value = $(this).val();
						if (value) {
							avail_langs.push(value);
						}
					});
					if ($.inArray(atts['lang'], avail_langs) == -1) {
						atts['lang'] = s.fallback_lang;
					}
					
					// Validate the attributes
					atts = me.validate(atts);
					
					// Load in attributes, add prefix
					for (var att in atts) {
						var setting = $('#' + gs.prefix + att + '.' + gs.setting);
						var value = atts[att];
						me.settingValue(setting, value);
						// Update highlights 
						setting.change();
						// If global setting changes and we access settings, it should declare loaded settings as changed even if they equal the global value, just so they aren't lost on save
						if (!setting.hasClass(gs.special)) {
							setting.addClass(gs.changed);
							if (setting.is('input[type=checkbox]')) {
			    				highlight = setting.next('span');
			    				highlight.addClass(gs.changed);
			    			}
						}
						console_log('loaded: ' + att + ':' + value);
					}
					
					editing = true;
					me.setSubmitText(s.submit_edit);
					
					// Code
					var content = currCrayon.html();
					if (inputHTML == 'encode') {
						content = crayon_encode_html(content); 
					} else if (inputHTML == 'decode') {
						content = crayon_decode_html(content);
					}
					code.val(content);
					
				} else {
					console_log('cannot load currNode of type pre');
				}
			} else {
				// We are creating a new Crayon, not editing
				editing = false;
				me.setSubmitText(s.submit_add);
				currCrayon = null;
				currClasses = '';
			}
			
			// Inline
			var inline = $('#' + s.inline_css);
			inline.change(function() {
				is_inline = $(this).is(':checked');
				var inline_hide = $('.' + s.inline_hide_css);
				var inline_single = $('.' + s.inline_hide_only_css);
				var disabled = [s.mark_css, s.range_css, s.title_css, s.url_css];
				
				for (var i in disabled) {
					var obj = $(disabled[i]);
					obj.attr('disabled', is_inline);
				}
				
				if (is_inline) {
					inline_hide.hide();
					inline_single.hide();
					inline_hide.closest('tr').hide();
					for (var i in disabled) {
						var obj = $(disabled[i]);
						obj.addClass('crayon-disabled');
					}
				} else {
					inline_hide.show();
					inline_single.show();
					inline_hide.closest('tr').show();
					for (var i in disabled) {
						var obj = $(disabled[i]);
						obj.removeClass('crayon-disabled');
					}
				}
			});
			inline.change();
			
			// Show the dialog
			var dialog_title = editing ? s.dialog_title_edit : s.dialog_title_add;
			$(s.dialog_title_css).html(dialog_title);
			if (showCallback) {
				showCallback();
			}
			
	    	code.focus();
	    	code_refresh();
	    	url_refresh();
	    	if (ajax_class_timer) {
	    		clearInterval(ajax_class_timer);
	    		ajax_class_timer_count = 0;
	    	}
	    	
	    	var ajax_window = $('#TB_window');
	    	ajax_window.hide();
	    	var fallback = function () {
	    		ajax_window.show();
	    		// Prevent draw artifacts
	    		var oldScroll = $(window).scrollTop();
	    		$(window).scrollTop(oldScroll+10);
	    		$(window).scrollTop(oldScroll-10);
	    	};
	    	
	    	ajax_class_timer = setInterval(function () {
	        	if ( typeof ajax_window != 'undefined' && !ajax_window.hasClass('crayon-te-ajax') ) {
	        		ajax_window.addClass('crayon-te-ajax');
	        		clearInterval(ajax_class_timer);
	        		fallback();
	        	}
	        	if (ajax_class_timer_count >= 100) {
	        		// In case it never loads, terminate
	        		clearInterval(ajax_class_timer);
	        		fallback();
	        	}
	        	ajax_class_timer_count++;
	    	}, 40);
	    };
	    
	    // XXX Add Crayon to editor
	    base.addCrayon = function() {
	    	var url = $(s.url_css);
	    	if (url.val().length == 0 && code.val().length == 0) {
				code.addClass(gs.selected);
				code.focus();
				return false;
	    	}
	    	code.removeClass(gs.selected);
			
			// Add inline for matching with CSS
			var inline = $('#' + s.inline_css);
			is_inline = inline.length != 0 && inline.is(':checked');
			
			// Spacing only for <pre>
			var br_before = br_after = '';
			if (!editing) {
				// Don't add spaces if editing
				if (!is_inline) {
					if (editor_name == 'html') {
						br_after = br_before = ' \n'; 
					} else {
						br_after = '<p>&nbsp;</p>';
					}
				} else {
					// Add a space after
					if (editor_name == 'html') {
						br_after = br_before = ' '; 
					} else {
						br_after = '&nbsp;';
					}
				}
			}
			
			var tag = (is_inline ? 'span' : 'pre');
			var shortcode = br_before + '<' + tag + ' ';
			
			var atts = {};
			shortcode += 'class="';
			
			var inline_re = new RegExp('\\b' + s.inline_css + '\\b', 'gim');
			if (is_inline) {
				// If don't have inline class, add it
				if (inline_re.exec(currClasses) == null) {
					currClasses += ' ' + s.inline_css + ' ';
				}
			} else {
				// Remove inline css if it exists
				currClasses = currClasses.replace(inline_re,'');
			}
			
			// Grab settings as attributes
			$('.'+gs.changed+'[id],.'+gs.changed+'['+s.data_value+']').each(function() {
	    		var id = $(this).attr('id');
	    		var value = $(this).attr(s.data_value);
	    		// Remove prefix
	    		id = util.removePrefixFromID(id);
	    		atts[id] = value;
	    	});
			
			// Settings
			atts['lang'] = $(s.lang_css).val();
			var mark = $(s.mark_css).val();
			if (mark.length != 0 && !is_inline) {
				atts['mark'] = mark;
			}
			var range = $(s.range_css).val();
			if (range.length != 0 && !is_inline) {
				atts['range'] = range;
			}
			
			// XXX Code highlighting, checked means 0!
			if ($(s.hl_css).is(':checked')) {
				atts['highlight'] = '0';
			}
			
			// XXX Very important when working with editor
	    	atts['decode'] = 'true';
	    	
	    	// Validate the attributes
			atts = me.validate(atts);
	    	
			for (var id in atts) {
	    		// Remove prefix, if exists
	    		var value = atts[id];
	    		console_log('add '+id+':'+value);
				shortcode += id + s.attr_sep + value + ' ';
			}
			
			// Add classes
			shortcode += currClasses;
			// Don't forget to close quote for class
			shortcode += '" ';
			
			if (!is_inline) {
				// Title
				var title = $(s.title_css).val();
				if (title.length != 0) {
					shortcode += 'title="' + title + '" ';
				}
				// URL
				var url = $(s.url_css).val();
				if (url.length != 0) {
					shortcode += 'data-url="' + url + '" ';
				}
			}
			
			var content = $(s.code_css).val();
			if (outputHTML == 'encode') {
				content = crayon_encode_html(content); 
			} else if (outputHTML == 'decode') {
				content = crayon_decode_html(content);
			}
			content = typeof content != 'undefined' ? content : '';
			shortcode += '>' + content + '</' + tag + '>' + br_after;
			
			if (editing) {
				// Edit the current selected node, update refere
				/*currPre =*/ editCallback(shortcode);
			} else {
				// Insert the tag and hide dialog
				insertCallback(shortcode);
			}
			
			return true;
		};
		
		base.submitButton = function() {
			console_log('submit');
			if (me.addCrayon() != false) {
				me.hideDialog();
			}
		};
		
		base.hideDialog = function() {
			console_log('hide');
			if (hideCallback) {
				hideCallback();
			}
		};
		
		// XXX Auxiliary methods
		
		base.setOrigValues = function() {
			$('.'+gs.setting+'[id]').each(function() {
				var setting = $(this);
				setting.attr(gs.orig_value, me.settingValue(setting));
			});
		};
		
		base.resetSettings = function() {
			console_log('reset');
			$('.'+gs.setting).each(function() {
				var setting = $(this);
				me.settingValue(setting, setting.attr(gs.orig_value));
				// Update highlights
				setting.change();
			});
			code.val('');
		};
		
		base.settingValue = function(setting, value) {
			if (typeof value == 'undefined') {
				// getter
				value = '';
				if (setting.is('input[type=checkbox]')) {
					// Boolean is stored as string
					value = setting.is(':checked') ? 'true' : 'false'; 
				} else {
					value = setting.val();
				}
				return value;
			} else {
				// setter
				if (setting.is('input[type=checkbox]')) {
					if (typeof value == 'string') {
						if (value == 'true' || value == '1') {
							value = true;
						} else if (value == 'false' || value == '0') {
							value = false;
						}
					}
					setting.prop('checked', value);
				} else {
					setting.val(value);
				}
				setting.attr(s.data_value, value);
			}
		};
		
		base.validate = function(atts) {
			var fields = ['range', 'mark'];
			for (var i in fields) {
				var field = fields[i];
				if (typeof atts[field] != 'undefined') {
					atts[field] = atts[field].replace(/\s/g, '');
				}
			}
			return atts;
		};
		
		base.isCrayon = function(node) {
			return node != null &&
				(node.nodeName == 'PRE' || (node.nodeName == 'SPAN' && $(node).hasClass(s.inline_css)));
		};
		
		base.elemValue = function(obj) {
			var value = null;
			if (obj.is('input[type=checkbox]')) {
				value = obj.is(':checked');
			} else {
				value = obj.val();
			}
			return value;
		};
		
		base.setSubmitText = function(text) {
			submit.html(text);
		}; 
		
	};
})(jQueryCrayon);
